library odoo_api;

import 'odoo_api_connector.dart';
import 'odoo_user_response.dart';

class OdooClient extends OdooConnector {
  OdooClient(String serverURL) : super(serverURL);

  // Get session information
  void getSessionInfo([Function(OdooResponse) callback]) {
    var url = createPath("/web/session/get_session_info");
    newRequest(url, createPayload({}), callback);
  }

  // Authenticate user
  void authenticate(String username, String password, String database,
      Function(AuthenticateCallback) callback) {
    var url = createPath("/web/session/authenticate");
    var params = {
      "db": database,
      "login": username,
      "password": password,
      "context": {}
    };
    newRequest(url, createPayload(params), (result) {
      getSessionInfo((session) {
        callback(new AuthenticateCallback(
            !result.hasError(), result, session.getResult()["session_id"]));
      });
    });
  }

  // Read records with given ids and fields
  void read(String model, List<int> ids, List<String> fields,
      Function(OdooResponse) callback,
      [dynamic kwargs, Map context]) {
    callKW(model, "read", [ids, fields], callback, kwargs, context);
  }

  // Search and read based on domain filters
  void searchRead(String model, List domain, List<String> fields,
      Function(OdooResponse) callback,
      [int offset = 0, int limit = 0, String order = "create_date"]) {
    var url = createPath("/web/dataset/search_read");
    var params = {
      "model": model,
      "fields": fields,
      "domain": domain,
      "context": {},
      "offset": offset,
      "limit": limit,
      "sort": order
    };
    newRequest(url, createPayload(params), callback);
  }

  // Call any model method with arguments
  void callKW(
      String model, String method, List args, Function(OdooResponse) callback,
      [dynamic kwargs, Map context]) {
    kwargs = kwargs == null ? {} : kwargs;
    context = context == null ? {} : context;
    var url = createPath("/web/dataset/call_kw/" + model + "/" + method);
    var params = {
      "model": model,
      "method": method,
      "args": args,
      "kwargs": kwargs,
      "context": context
    };
    newRequest(url, createPayload(params), callback);
  }

  // Create new record for model
  void create(String model, Map values, Function(OdooResponse) callback) {
    callKW(model, "create", [values], callback);
  }

  // Write record with ids and values
  void write(
      String model, List ids, Map values, Function(OdooResponse) callback) {
    callKW(model, "write", [ids, values], callback);
  }

  // Remove record from system
  void unlink(String model, List ids, Function(OdooResponse) callback) {
    callKW(model, "unlink", [ids], callback);
  }

  // Call json controller
  void callController(
      String path, Map params, Function(OdooResponse) callback) {
    newRequest(createPath(path), createPayload(params), callback);
  }
}
